@@ -26,6 +26,7 @@ LDAP_BASE = config[:base]
 LDAP_PREFIX = options.prefix
 LDAP_USER = config[:bind_dn]
 LDAP_PASSWORD = config[:password]
+LDAP_METHOD = config[:method]
 
 class ALUser < ActiveLdap::Base
   ldap_mapping :dn_attribute => 'uid', :prefix => LDAP_PREFIX,
@@ -74,7 +75,11 @@ end
 
 def ldap_connection
   require 'ldap'
-  conn = LDAP::Conn.new(LDAP_HOST, LDAP_PORT)
+  if LDAP_METHOD == :tls
+    conn = LDAP::SSLConn.new(LDAP_HOST, LDAP_PORT, true)
+  else
+    conn = LDAP::Conn.new(LDAP_HOST, LDAP_PORT)
+  end
   conn.set_option(LDAP::LDAP_OPT_PROTOCOL_VERSION, 3)
   conn.bind(LDAP_USER, LDAP_PASSWORD) if LDAP_USER and LDAP_PASSWORD
   conn
@@ -87,6 +92,7 @@ def net_ldap_connection
   net_ldap_conn = Net::LDAP::Connection.new(:host => LDAP_HOST,
                                             :port => LDAP_PORT)
   if LDAP_USER and LDAP_PASSWORD
+    net_ldap_conn.setup_encryption(:method => :start_tls) if LDAP_METHOD == :tls
     net_ldap_conn.bind(:method => :simple,
                        :username => LDAP_USER,
                        :password => LDAP_PASSWORD)
@@ -166,19 +172,23 @@ def main(do_populate)
   al_count_without_object_creation = 0
   ldap_count = 0
   net_ldap_count = 0
-  Benchmark.bm(20) do |x|
+  Benchmark.bmbm(20) do |x|
     [1].each do |n|
+      GC.start
       x.report("%3dx: AL" % n) {n.times {al_count = search_al}}
+      GC.start
       x.report("%3dx: AL(No Obj)" % n) do
         n.times do
           al_count_without_object_creation = search_al_without_object_creation
         end
       end
+      GC.start
       if ldap_conn
         x.report("%3dx: LDAP" % n) do
           n.times {ldap_count = search_ldap(ldap_conn)}
         end
       end
+      GC.start
       if net_ldap_conn
         x.report("%3dx: Net::LDAP" % n) do
           n.times {net_ldap_count = search_net_ldap(net_ldap_conn)}
