@@ -19,8 +19,14 @@ class UsersController < ApplicationController
   def update
     @user = find(params[:id])
     previous_user_password = @user.user_password
-    @user.replace_class(params["object-classes"])
-    if @user.update_attributes(params[:user])
+    object_class_error_message = nil
+    begin
+      @user.replace_class(params["object-classes"])
+    rescue ActiveLdap::RequiredObjectClassMissed
+      object_class_error_message = $!.message
+    end
+    if @user.update_attributes(params[:user]) and
+        object_class_error_message.nil?
       if previous_user_password != @user.user_password and @user.connected?
         @user.bind(@user.password)
       end
@@ -28,20 +34,29 @@ class UsersController < ApplicationController
       redirect_to :action => 'show', :id => @user
     else
       @user.password = @user.password_confirmation = nil
+      @user.errors.add("objectClass", object_class_error_message)
       render :action => 'edit'
     end
   end
 
   def update_object_classes
     @user = find(params[:id])
-    @user.replace_class(params["object-classes"])
+    begin
+      @user.replace_class(params["object-classes"])
+    rescue ActiveLdap::RequiredObjectClassMissed
+      flash.now[:inline_notice] = $!.message
+      erb = "<%= flash_box(flash[:inline_notice], :need_container => true) %>"
+      render(:inline => erb, :status => 400)
+      return
+    end
     available_attributes = @user.attribute_names(true)
     attributes = {}
     (params[:user] || {}).each do |key, value|
       attributes[key] = value if available_attributes.include?(key)
     end
     @user.attributes = attributes
-    render(:partial => "attributes_update_form")
+    erb = "<div>\n<%= render(:partial => 'attributes_update_form') %>\n</div>"
+    render(:inline => erb)
   end
 
   private
