@@ -2,9 +2,11 @@ class WelcomeController < ApplicationController
   include UrlHelper
 
   def index
-    unless logged_in?
+    if Entry.empty?
+      redirect_to(populate_path)
+    elsif !logged_in?
       flash.keep(:notice)
-      redirect_to(:login_path)
+      redirect_to(login_path)
     end
   end
 end
