@@ -192,9 +192,9 @@
 #
 # Under ou=People I store user objects, and under ou=Groups, I store group
 # objects.  What |ldap_mapping| has done is mapped the class in to the LDAP tree
-# abstractly. With the given :dnattr and :prefix, it will only work for entries
-# under ou=Groups,dc=dataspill,dc=org using the primary attribute 'cn' as the
-# beginning of the distinguished name.
+# abstractly. With the given :dn_attributes and :prefix, it will only work for
+# entries under ou=Groups,dc=dataspill,dc=org using the primary attribute 'cn'
+# as the beginning of the distinguished name.
 #
 # Just for clarity, here's how the arguments map out:
 #
@@ -205,7 +205,8 @@
 #                :base from configuration.rb
 #
 # :scope tells ActiveLdap to only search under ou=Groups, and not to look deeper
-# for dnattr matches. (e.g. cn=develop,ou=DevGroups,ou=Groups,dc=dataspill,dc=org)
+# for dn_attribute matches.
+# (e.g. cn=develop,ou=DevGroups,ou=Groups,dc=dataspill,dc=org)
 #
 # Something's missing: :classes.  :classes is used to tell Ruby/ActiveLdap what
 # the minimum requirement is when creating a new object. LDAP uses objectClasses
@@ -293,7 +294,7 @@
 # mind, the above definition could become:
 #
 #   irb> class User < ActiveLdap::Base
-#   irb*   ldap_mapping :dnattr => 'uid', :prefix => 'People', :classes => ['top','account']
+#   irb*   ldap_mapping :dn_attribute => 'uid', :prefix => 'People', :classes => ['top','account']
 #   irb*   belongs_to :groups, :class => 'Group', :many => 'memberUid'
 #   irb* end
 #
@@ -348,8 +349,8 @@
 #   => "root"
 #
 # In this simple example, Group.find took the search string of 'deve*' and
-# searched for the first match in Group where the dnattr matched the query. This
-# is the simplest example of .find.
+# searched for the first match in Group where the dn_attribute matched the
+# query. This is the simplest example of .find.
 #
 #   irb> Group.find(:all, '*').collect {|group| group.cn}
 #   => ["root", "daemon", "bin", "sys", "adm", "tty", ..., "develop"]
@@ -916,12 +917,18 @@ end
 
 require_gem_if_need.call("active_support", "activesupport")
 
-if Dependencies.respond_to?(:load_paths)
-  Dependencies.load_paths << File.expand_path(File.dirname(__FILE__))
+if ActiveSupport.const_defined?(:Dependencies)
+  dependencies = ActiveSupport::Dependencies
+else
+  dependencies = Dependencies
+end
+
+if dependencies.respond_to?(:load_paths)
+  dependencies.load_paths << File.expand_path(File.dirname(__FILE__))
 end
 
 module ActiveLdap
-  VERSION = "0.9.0"
+  VERSION = "1.0.1"
 end
 
 if RUBY_PLATFORM.match('linux')
@@ -939,6 +946,9 @@ require 'active_ldap/get_text'
 
 require 'active_ldap/base'
 
+require 'active_ldap/distinguished_name'
+require 'active_ldap/ldif'
+
 require 'active_ldap/associations'
 require 'active_ldap/attributes'
 require 'active_ldap/configuration'
@@ -949,7 +959,6 @@ require 'active_ldap/human_readable'
 
 require 'active_ldap/acts/tree'
 
-require 'active_ldap/distinguished_name'
 require 'active_ldap/populate'
 require 'active_ldap/escape'
 require 'active_ldap/helper'
@@ -974,7 +983,7 @@ ActiveLdap::Base.class_eval do
 end
 
 unless defined?(ACTIVE_LDAP_CONNECTION_ADAPTERS)
-  ACTIVE_LDAP_CONNECTION_ADAPTERS = %w(ldap net_ldap)
+  ACTIVE_LDAP_CONNECTION_ADAPTERS = %w(ldap net_ldap jndi)
 end
 
 ACTIVE_LDAP_CONNECTION_ADAPTERS.each do |adapter|
