@@ -13,19 +13,37 @@ module ActiveLdap
 
     class Ldap < Base
       module Method
-        class SSL
+        class Base
+          def ssl?
+            false
+          end
+
+          def start_tls?
+            false
+          end
+        end
+
+        class SSL < Base
           def connect(host, port)
             LDAP::SSLConn.new(host, port, false)
           end
-        end
 
-        class TLS
-          def connect(host, port)
-            LDAP::SSLConn.new(host, port, true)
+          def ssl?
+            true
           end
         end
 
-        class Plain
+        class TLS < Base
+          def connect(host, port)
+            LDAP::SSLConn.new(host, port, true)
+          end
+
+          def start_tls?
+            true
+          end
+        end
+
+        class Plain < Base
           def connect(host, port)
             LDAP::Conn.new(host, port)
           end
@@ -34,7 +52,11 @@ module ActiveLdap
 
       def connect(options={})
         super do |host, port, method|
-          method.connect(host, port)
+          uri = construct_uri(host, port, method.ssl?)
+          with_start_tls = method.start_tls?
+          info = {:uri => uri, :with_start_tls => with_start_tls}
+          [log("connect", info) {method.connect(host, port)},
+           uri, with_start_tls]
         end
       end
 
@@ -53,7 +75,7 @@ module ActiveLdap
 
       def bind_as_anonymous(options={})
         super do
-          execute(:bind)
+          execute(:bind, :name => "bind: anonymous")
           true
         end
       end
@@ -66,7 +88,11 @@ module ActiveLdap
         super(options) do |base, scope, filter, attrs, limit, callback|
           begin
             i = 0
-            execute(:search, base, scope, filter, attrs) do |entry|
+            info = {
+              :base => base, :scope => scope_name(scope),
+              :filter => filter, :attributes => attrs,
+            }
+            execute(:search, info, base, scope, filter, attrs) do |entry|
               i += 1
               attributes = {}
               entry.attrs.each do |attr|
@@ -76,6 +102,11 @@ module ActiveLdap
               break if limit and limit <= i
             end
           rescue RuntimeError
+            begin
+              @connection.assert_error_code
+            rescue LDAP::ServerDown
+              raise ConnectionError, $!.message
+            end
             if $!.message == "no result returned by search"
               @logger.debug do
                 args = [filter, attrs.inspect]
@@ -88,37 +119,55 @@ module ActiveLdap
         end
       end
 
-      def to_ldif(dn, attributes)
-        ldif = LDAP::LDIF.to_ldif("dn", [dn.dup])
-        attributes.sort_by do |key, value|
-          key
-        end.each do |key, values|
-          ldif << LDAP::LDIF.to_ldif(key, values)
-        end
-        ldif
-      end
-
-      def load(ldifs, options={})
-        super do |ldif|
-          LDAP::LDIF.parse_entry(ldif).send(@connection)
-        end
-      end
-
       def delete(targets, options={})
         super do |target|
-          execute(:delete, target)
+          controls = options[:controls]
+          info = {:dn => target}
+          if controls
+            info.merge!(:name => :delete, :controls => controls)
+            execute(:delete_ext, info,
+                    target, controls, [])
+          else
+            execute(:delete, info, target)
+          end
         end
       end
 
       def add(dn, entries, options={})
         super do |dn, entries|
-          execute(:add, dn, parse_entries(entries))
+          controls = options[:controls]
+          attributes = parse_entries(entries)
+          info = {:dn => dn, :attributes => entries}
+          if controls
+            info.merge!(:name => :add, :controls => controls)
+            execute(:add_ext, info, dn, attributes, controls, [])
+          else
+            execute(:add, info, dn, attributes)
+          end
         end
       end
 
       def modify(dn, entries, options={})
         super do |dn, entries|
-          execute(:modify, dn, parse_entries(entries))
+          controls = options[:controls]
+          attributes = parse_entries(entries)
+          info = {:dn => dn, :attributes => entries}
+          if controls
+            info.merge!(:name => :modify, :controls => controls)
+            execute(:modify_ext, info, dn, attributes, controls, [])
+          else
+            execute(:modify, info, dn, attributes)
+          end
+        end
+      end
+
+      def modify_rdn(dn, new_rdn, delete_old_rdn, new_superior, options={})
+        super do |dn, new_rdn, delete_old_rdn, new_superior|
+          info = {
+            :name => "modify: RDN",
+            :dn => dn, :new_rdn => new_rdn, :delete_old_rdn => delete_old_rdn
+          }
+          execute(:modrdn, info, dn, new_rdn, delete_old_rdn)
         end
       end
 
@@ -129,31 +178,16 @@ module ActiveLdap
         end
       end
 
-      def root_dse(attributes, options={})
-        sec = options[:sec] || 0
-        usec = options[:usec] || 0
-        @connection.root_dse(attributes, sec, usec)
-      end
-
-      def execute(method, *args, &block)
+      def execute(method, info=nil, *args, &block)
         begin
-          @connection.send(method, *args, &block)
+          name = (info || {}).delete(:name) || method
+          log(name, info) {@connection.send(method, *args, &block)}
         rescue LDAP::ResultError
           @connection.assert_error_code
           raise $!.message
         end
       end
 
-      def with_timeout(try_reconnect=true, options={}, &block)
-        begin
-          super
-        rescue LDAP::ServerDown => e
-          @logger.error {_("LDAP server is down: %s") % e.message}
-          retry if try_reconnect and reconnect(options)
-          raise ConnectionError.new(e.message)
-        end
-      end
-
       def ensure_method(method)
         Method.constants.each do |name|
           if method.to_s.downcase == name.downcase
@@ -183,6 +217,14 @@ module ActiveLdap
         value
       end
 
+      def scope_name(scope)
+        {
+          LDAP::LDAP_SCOPE_BASE => :base,
+          LDAP::LDAP_SCOPE_SUBTREE => :sub,
+          LDAP::LDAP_SCOPE_ONELEVEL => :one,
+        }[scope]
+      end
+
       def sasl_bind(bind_dn, options={})
         super do |bind_dn, mechanism, quiet|
           begin
@@ -192,7 +234,10 @@ module ActiveLdap
             if need_credential_sasl_mechanism?(mechanism)
               args << password(bind_dn, options)
             end
-            execute(:sasl_bind, *args)
+            info = {
+              :name => "bind: SASL", :dn => bind_dn, :mechanism => mechanism
+            }
+            execute(:sasl_bind, info, *args)
           ensure
             @connection.sasl_quiet = sasl_quiet
           end
@@ -201,7 +246,7 @@ module ActiveLdap
 
       def simple_bind(bind_dn, options={})
         super do |bind_dn, passwd|
-          execute(:bind, bind_dn, passwd)
+          execute(:bind, {:dn => bind_dn}, bind_dn, passwd)
         end
       end
 
@@ -220,7 +265,7 @@ module ActiveLdap
 
       def ensure_mod_type(type)
         case type
-        when :replace, :add
+        when :replace, :add, :delete
           LDAP.const_get("LDAP_MOD_#{type.to_s.upcase}")
         else
           raise ArgumentError, _("unknown type: %s") % type
