@@ -6,7 +6,11 @@ module ActiveLdap
       private
       def insert_entry(entry)
         old_value = entry[@options[:many], true]
-        new_value = old_value + @owner[@options[:foreign_key_name], true]
+        foreign_key_name = @options[:foreign_key_name]
+        if foreign_key_name == "dn"
+          old_value = dn_values_to_string_values(old_value)
+        end
+        new_value = old_value + @owner[foreign_key_name, true]
         new_value = new_value.uniq.sort
         if old_value != new_value
           entry[@options[:many]] = new_value
@@ -17,7 +21,11 @@ module ActiveLdap
       def delete_entries(entries)
         entries.each do |entry|
           old_value = entry[@options[:many], true]
-          new_value = old_value - @owner[@options[:foreign_key_name], true]
+          foreign_key_name = @options[:foreign_key_name]
+          if foreign_key_name == "dn"
+            old_value = dn_values_to_string_values(old_value)
+          end
+          new_value = old_value - @owner[foreign_key_name, true]
           new_value = new_value.uniq.sort
           if old_value != new_value
             entry[@options[:many]] = new_value
