@@ -78,6 +78,16 @@ module ActiveLdap
 
         result && self
       end
+
+      def dn_values_to_string_values(values)
+        values.collect do |value|
+          if value.is_a?(DN)
+            value.to_s
+          else
+            value
+          end
+        end
+      end
     end
   end
 end
