@@ -1,8 +1,11 @@
 require 'active_ldap/association/collection'
+require 'active_ldap/association/has_many_utils'
 
 module ActiveLdap
   module Association
     class HasMany < Collection
+      include HasManyUtils
+
       private
       def insert_entry(entry)
         entry[primary_key] = @owner[@options[:foreign_key_name]]
@@ -10,19 +13,7 @@ module ActiveLdap
       end
 
       def find_target
-        foreign_base_key = primary_key
-        components = @owner[@options[:foreign_key_name], true].collect do |value|
-          key = val = nil
-          if foreign_base_key == "dn"
-            key, val = value.split(",")[0].split("=") unless value.empty?
-          else
-            key, val = foreign_base_key, value
-          end
-          [key, val]
-        end.reject do |key, val|
-          key.nil? or val.nil?
-        end
-        foreign_class.find(:all, :filter => [:or, *components])
+        collect_targets(:foreign_key_name)
       end
 
       def delete_entries(entries)
