@@ -105,6 +105,14 @@ module ActiveLdap
   class StrongAuthenticationRequired < Error
   end
 
+  class DistinguishedNameInputInvalid < Error
+    attr_reader :input
+    def initialize(input=nil)
+      @input = input
+      super(_("invalid distinguished name (DN) to parse: %s") % @input.inspect)
+    end
+  end
+
   class DistinguishedNameInvalid < Error
     attr_reader :dn, :reason
     def initialize(dn, reason=nil)
@@ -123,16 +131,63 @@ module ActiveLdap
   end
 
   class LdifInvalid < Error
-    attr_reader :ldif, :reason
-    def initialize(ldif, reason=nil)
+    attr_reader :ldif, :reason, :line, :column, :nearest
+    def initialize(ldif, reason=nil, line=nil, column=nil)
       @ldif = ldif
       @reason = reason
+      @line = line
+      @column = column
+      @nearest = nil
       if @reason
-        message = _("%s is invalid LDIF: %s") % [@ldif, @reason]
+        message = _("invalid LDIF: %s:") % @reason
       else
-        message = _("%s is invalid LDIF") % @ldif
+        message = _("invalid LDIF:")
       end
-      super(message)
+      if @line and @column
+        @nearest = detect_nearest(@line, @column)
+        snippet = generate_snippet
+        message << "\n#{snippet}\n"
+      end
+      super("#{message}\n#{numbered_ldif}")
+    end
+
+    NEAREST_MARK = "|@|"
+    private
+    def detect_nearest(line, column)
+      nearest = @ldif.to_a[line - 1] || ""
+      if column - 1 == nearest.size # for JRuby 1.0.2 :<
+        nearest << NEAREST_MARK
+      else
+        nearest[column - 1, 0] = NEAREST_MARK
+      end
+      nearest = "#{@ldif.to_a[line - 2]}#{nearest}" if nearest == NEAREST_MARK
+      nearest
+    end
+
+    def generate_snippet
+      nearest = @nearest.chomp
+      column_column = ":#{@column}"
+      target_position_info = "#{@line}#{column_column}: "
+      if /\n/ =~ nearest
+        snippet = "%#{Math.log10(@line).truncate}d" % (@line - 1)
+        snippet << " " * column_column.size
+        snippet << ": "
+        snippet << nearest.gsub(/\n/, "\n#{target_position_info}")
+      else
+        snippet = "#{target_position_info}#{nearest}"
+      end
+      snippet
+    end
+
+    def numbered_ldif
+      return @ldif if @ldif.blank?
+      lines = @ldif.to_a
+      format = "%#{Math.log10(lines.size).truncate + 1}d: %s"
+      i = 0
+      lines.collect do |line|
+        i += 1
+        format % [i, line]
+      end.join
     end
   end
 
@@ -173,6 +228,15 @@ module ActiveLdap
     end
   end
 
+  class AttributeValueInvalid < Error
+    attr_reader :attribute, :value
+    def initialize(attribute, value, message)
+      @attribute = attribute
+      @value = value
+      super(message)
+    end
+  end
+
   # Base
   #
   # Base is the primary class which contains all of the core
@@ -190,9 +254,12 @@ module ActiveLdap
       end
     end
 
+    cattr_accessor :colorize_logging, :instance_writer => false
+    @@colorize_logging = true
+
     VALID_LDAP_MAPPING_OPTIONS = [:dn_attribute, :prefix, :scope,
                                   :classes, :recommended_classes,
-                                  :sort_by, :order]
+                                  :excluded_classes, :sort_by, :order]
 
     cattr_accessor :logger
     cattr_accessor :configurations
@@ -228,11 +295,19 @@ module ActiveLdap
     class_local_attr_accessor false, :prefix, :base
     class_local_attr_accessor true, :dn_attribute, :scope, :sort_by, :order
     class_local_attr_accessor true, :required_classes, :recommended_classes
+    class_local_attr_accessor true, :excluded_classes
 
     class << self
       # Hide new in Base
       private :new
 
+      def inherited(sub_class)
+        super
+        sub_class.module_eval do
+          include GetTextSupport
+        end
+      end
+
       # Connect and bind to LDAP creating a class variable for use by
       # all ActiveLdap objects.
       #
@@ -293,10 +368,12 @@ module ActiveLdap
         validate_ldap_mapping_options(options)
 
         self.dn_attribute = options[:dn_attribute] || default_dn_attribute
+        self.dn_attribute = dn_attribute.to_s if dn_attribute.is_a?(Symbol)
         self.prefix = options[:prefix] || default_prefix
         self.scope = options[:scope]
         self.required_classes = options[:classes]
         self.recommended_classes = options[:recommended_classes]
+        self.excluded_classes = options[:excluded_classes]
         self.sort_by = options[:sort_by]
         self.order = options[:order]
 
@@ -323,6 +400,16 @@ module ActiveLdap
         end.join(",")
       end
 
+      alias_method :base_without_parsed_cache_clear=, :base=
+      def base=(value)
+        self.base_without_parsed_cache_clear = value
+        @parsed_base = nil
+      end
+
+      def parsed_base
+        @parsed_base ||= DN.parse(base)
+      end
+
       alias_method :scope_without_validation=, :scope=
       def scope=(scope)
         validate_scope(scope)
@@ -348,7 +435,52 @@ module ActiveLdap
         dn_attribute
       end
 
+      def inspect
+        if self == Base
+          super
+        else
+          class_names = []
+          must = []
+          may = []
+          class_names = classes.collect do |object_class|
+            must.concat(object_class.must)
+            may.concat(object_class.may)
+            object_class.name
+          end
+          detail = ["objectClass:<#{class_names.join(', ')}>",
+                    "must:<#{inspect_attributes(must)}>",
+                    "may:<#{inspect_attributes(may)}>"].join(", ")
+          "#{super}(#{detail})"
+        end
+      end
+
       private
+      def inspect_attributes(attributes)
+	inspected_attribute_names = {}
+        attributes.collect do |attribute|
+          if inspected_attribute_names.has_key?(attribute.name)
+            nil
+          else
+            inspected_attribute_names[attribute.name] = true
+            inspect_attribute(attribute)
+          end
+        end.compact.join(', ')
+      end
+
+      def inspect_attribute(attribute)
+        syntax = attribute.syntax
+        result = "#{attribute.name}"
+        if syntax and !syntax.description.blank?
+          result << ": #{syntax.description}"
+        end
+        properties = []
+        properties << "read-only" if attribute.read_only?
+        properties << "binary" if attribute.binary?
+        properties << "binary-required" if attribute.binary_required?
+        result << "(#{properties.join(', ')})" unless properties.empty?
+        result
+      end
+
       def validate_ldap_mapping_options(options)
         options.assert_valid_keys(VALID_LDAP_MAPPING_OPTIONS)
       end
@@ -394,7 +526,7 @@ module ActiveLdap
           end
           dn_attribute || "cn"
         else
-          Inflector.underscore(Inflector.demodulize(name))
+          name.demodulize.underscore
         end
       end
 
@@ -402,7 +534,7 @@ module ActiveLdap
         if name.empty?
           nil
         else
-          "ou=#{Inflector.pluralize(Inflector.demodulize(name))}"
+          "ou=#{name.demodulize.pluralize}"
         end
       end
     end
@@ -410,6 +542,7 @@ module ActiveLdap
     self.scope = :sub
     self.required_classes = ['top']
     self.recommended_classes = []
+    self.excluded_classes = []
 
     include Enumerable
 
@@ -424,25 +557,26 @@ module ActiveLdap
       init_base
       @new_entry = true
       initial_classes = required_classes | recommended_classes
-      if attributes.nil?
-        apply_object_class(initial_classes)
-      elsif attributes.is_a?(String) or attributes.is_a?(Array)
-        apply_object_class(initial_classes)
+      case attributes
+      when nil
+        self.classes = initial_classes
+      when String, Array, DN
+        self.classes = initial_classes
         self.dn = attributes
-      elsif attributes.is_a?(Hash)
+      when Hash
         classes, attributes = extract_object_class(attributes)
-        apply_object_class(classes | initial_classes)
+        self.classes = classes | initial_classes
         normalized_attributes = {}
         attributes.each do |key, value|
           real_key = to_real_attribute_name(key) || key
           normalized_attributes[real_key] = value
         end
-        self.dn = normalized_attributes[dn_attribute]
+        self.dn = normalized_attributes.delete(dn_attribute)
         self.attributes = normalized_attributes
       else
-        message = _("'%s' must be either nil, DN value as String or Array " \
-                    "or attributes as Hash") % attributes.inspect
-        raise ArgumentError, message
+        format = _("'%s' must be either nil, DN value as ActiveLdap::DN, " \
+                   "String or Array or attributes as Hash")
+        raise ArgumentError, format % attributes.inspect
       end
       yield self if block_given?
       assert_dn_attribute
@@ -471,13 +605,11 @@ module ActiveLdap
     end
 
     def may
-      ensure_apply_object_class
-      @may
+      entry_attribute.may
     end
 
     def must
-      ensure_apply_object_class
-      @must
+      entry_attribute.must
     end
 
     # attributes
@@ -485,15 +617,7 @@ module ActiveLdap
     # Return attribute methods so that a program can determine available
     # attributes dynamically without schema awareness
     def attribute_names(normalize=false)
-      ensure_apply_object_class
-      names = @attribute_names.keys
-      if normalize
-        names.collect do |name|
-          to_real_attribute_name(name)
-        end.uniq
-      else
-        names
-      end
+      entry_attribute.names(normalize)
     end
 
     def attribute_present?(name)
@@ -520,16 +644,7 @@ module ActiveLdap
     #
     # Return the authoritative dn
     def dn
-      return base if @dn_is_base
-
-      dn_value = id
-      if dn_value.nil?
-        raise DistinguishedNameNotSetError.new,
-                _("%s's DN attribute (%s) isn't set") % [self, dn_attribute]
-      end
-      _base = base
-      _base = nil if _base.empty?
-      ["#{dn_attribute}=#{dn_value}", _base].compact.join(",")
+      @dn ||= compute_dn
     end
 
     def id
@@ -542,12 +657,14 @@ module ActiveLdap
 
     def dn=(value)
       set_attribute(dn_attribute, value)
+      @dn = nil
     end
     alias_method(:id=, :dn=)
 
     alias_method(:dn_attribute_of_class, :dn_attribute)
     def dn_attribute
-      @dn_attribute || dn_attribute_of_class
+      _dn_attribute = @dn_attribute || dn_attribute_of_class
+      to_real_attribute_name(_dn_attribute) || _dn_attribute
     end
 
     def default_search_attribute
@@ -593,8 +710,6 @@ module ActiveLdap
     #       using class_eval instead of using method_missing.  This would
     #       give tab completion in irb.
     def method_missing(name, *args, &block)
-      ensure_apply_object_class
-
       key = name.to_s
       case key
       when /=$/
@@ -629,9 +744,8 @@ module ActiveLdap
 
     # Add available attributes to the methods
     def methods(inherited_too=true)
-      ensure_apply_object_class
-      target_names = @attribute_names.keys + @attribute_aliases.keys
-      target_names -= ['objectClass', Inflector.underscore('objectClass')]
+      target_names = entry_attribute.all_names
+      target_names -= ['objectClass', 'objectClass'.underscore]
       super + target_names.uniq.collect do |x|
         [x, "#{x}=", "#{x}?", "#{x}_before_type_cast"]
       end.flatten
@@ -639,10 +753,12 @@ module ActiveLdap
 
     alias_method :respond_to_without_attributes?, :respond_to?
     def respond_to?(name, include_priv=false)
-      have_attribute?(name.to_s) or
-        (/(?:=|\?|_before_type_cast)$/ =~ name.to_s and
-         have_attribute?($PREMATCH)) or
-        super
+      return true if super
+
+      name = name.to_s
+      return true if have_attribute?(name)
+      return false if /(?:=|\?|_before_type_cast)$/ !~ name
+      have_attribute?($PREMATCH)
     end
 
     # Updates a given attribute and saves immediately
@@ -677,7 +793,7 @@ module ActiveLdap
     # Do not let URL/form hackers supply the keys.
     def attributes=(new_attributes)
       return if new_attributes.nil?
-      _schema = nil
+      _schema = _local_entry_attribute = nil
       targets = remove_attributes_protected_from_mass_assignment(new_attributes)
       targets.each do |key, value|
         setter = "#{key}="
@@ -685,18 +801,23 @@ module ActiveLdap
           _schema ||= schema
           attribute = _schema.attribute(key)
           next if attribute.id.nil?
-          define_attribute_methods(attribute)
+          _local_entry_attribute ||= local_entry_attribute
+          _local_entry_attribute.register(attribute)
         end
         send(setter, value)
       end
     end
 
-    def to_ldif
+    def to_ldif_record
       super(dn, normalize_data(@data))
     end
 
+    def to_ldif
+      Ldif.new([to_ldif_record]).to_s
+    end
+
     def to_xml(options={})
-      root = options[:root] || Inflector.underscore(self.class.name)
+      root = options[:root] || self.class.name.underscore
       result = "<#{root}>\n"
       result << "  <dn>#{dn}</dn>\n"
       normalize_data(@data).sort_by {|key, values| key}.each do |key, values|
@@ -735,7 +856,7 @@ module ActiveLdap
 
       @ldap_data.update(attributes)
       classes, attributes = extract_object_class(attributes)
-      apply_object_class(classes)
+      self.classes = classes
       self.attributes = attributes
       @new_entry = false
       self
@@ -759,11 +880,9 @@ module ActiveLdap
       end
     end
 
-    def bind(config_or_password={}, &block)
+    def bind(config_or_password={}, config_or_ignore=nil, &block)
       if config_or_password.is_a?(String)
-        config = {:password => config_or_password}
-      elsif config_or_password.respond_to?(:call)
-        config = {:password_block => config_or_password}
+        config = (config_or_ignore || {}).merge(:password => config_or_password)
       else
         config = config_or_password
       end
@@ -776,7 +895,7 @@ module ActiveLdap
         @connection = nil
         connection.connect
         @connection = connection
-        @schema = nil
+        clear_connection_based_cache
         clear_association_cache
       rescue ActiveLdap::Error
         remove_connection
@@ -786,6 +905,17 @@ module ActiveLdap
       true
     end
 
+    def clear_connection_based_cache
+      @schema = nil
+      @local_entry_attribute = nil
+      clear_object_class_based_cache
+    end
+
+    def clear_object_class_based_cache
+      @entry_attribute = nil
+      @real_names = {}
+    end
+
     def schema
       @schema ||= super
     end
@@ -797,6 +927,7 @@ module ActiveLdap
 
     undef_method :base=
     def base=(object_local_base)
+      @dn = nil
       @base = object_local_base
     end
 
@@ -812,54 +943,60 @@ module ActiveLdap
     end
 
     def inspect
-      abbreviate_instance_variables do
-        super
-      end
-    end
-
-    def pretty_print(q)
-      abbreviate_instance_variables do
-        q.pp_object(self)
-      end
+      object_classes = entry_attribute.object_classes
+      inspected_object_classes = object_classes.collect do |object_class|
+        object_class.name
+      end.join(', ')
+      must_attributes = must.collect(&:name).sort.join(', ')
+      may_attributes = may.collect(&:name).sort.join(', ')
+      inspected_attributes = attribute_names.sort.collect do |name|
+        inspect_attribute(name)
+      end.join(', ')
+      result = "\#<#{self.class} objectClass:<#{inspected_object_classes}>, "
+      result << "must:<#{must_attributes}>, may:<#{may_attributes}>, "
+      result << "#{inspected_attributes}>"
+      result
     end
 
     private
-    def abbreviate_instance_variables
-      @abbreviating ||= nil
-      connection, @connection = @connection, nil
-      schema, @schema = @schema, nil
-      attribute_schemata, @attribute_schemata = @attribute_schemata, nil
-      must, may = @must, @may
-      object_classes = @object_classes
-      unless @abbreviating
-        @abbreviating = true
-        @must, @may = @must.collect(&:name), @may.collect(&:name)
-        @object_classes = @object_classes.collect(&:name)
+    def inspect_attribute(name)
+      values = get_attribute(name, true)
+      values.collect do |value|
+        if value.is_a?(String) and value.length > 50
+          "#{value[0, 50]}...".inspect
+        elsif value.is_a?(Date) || value.is_a?(Time)
+          "#{value.to_s(:db)}"
+        else
+          value.inspect
+        end
       end
-      yield
-    ensure
-      @connection = connection
-      @schema = schema
-      @attribute_schemata = attribute_schemata
-      @must = must
-      @may = may
-      @object_classes = object_classes
-      @abbreviating = false
+      "#{name}: #{values.inspect}"
+    end
+
+    def attribute_name_resolvable_without_connection?
+      @entry_attribute and @local_entry_attribute
+    end
+
+    def entry_attribute
+      @entry_attribute ||= connection.entry_attribute(@data["objectClass"] || [])
+    end
+
+    def local_entry_attribute
+      @local_entry_attribute ||= connection.entry_attribute([])
     end
 
     def extract_object_class(attributes)
       classes = []
-      attrs = attributes.stringify_keys.reject do |key, value|
-        if key == 'objectClass' or
-            key.underscore == 'object_class' or
-            key.downcase == 'objectclass'
-          classes |= [value].flatten
-          true
+      attrs = {}
+      attributes.each do |key, value|
+        key = key.to_s
+        if /\Aobject_?class\z/i =~ key
+          classes.concat(value.to_a)
         else
-          false
+          attrs[key] = value
         end
       end
-      [classes, attrs]
+      [classes, attributes]
     end
 
     def init_base
@@ -868,11 +1005,12 @@ module ActiveLdap
 
     def initialize_by_ldap_data(dn, attributes)
       init_base
+      @dn = dn
       @new_entry = false
       @dn_is_base = false
       @ldap_data = attributes
       classes, attributes = extract_object_class(attributes)
-      apply_object_class(classes)
+      self.classes = classes
       self.dn = dn
       self.attributes = attributes
       yield self if block_given?
@@ -893,32 +1031,22 @@ module ActiveLdap
 
     def to_real_attribute_name(name, allow_normalized_name=false)
       return name if name.nil?
-      ensure_apply_object_class
-      name = name.to_s
-      real_name = @attribute_names[name]
-      real_name ||= @attribute_aliases[Inflector.underscore(name)]
-      if real_name
-        real_name
-      elsif allow_normalized_name
-        @normalized_attribute_names[normalize_attribute_name(name)]
+      if allow_normalized_name
+        entry_attribute.normalize(name, allow_normalized_name) ||
+          local_entry_attribute.normalize(name, allow_normalized_name)
       else
-        nil
+        @real_names[name] ||=
+          entry_attribute.normalize(name, false) ||
+          local_entry_attribute.normalize(name, false)
       end
     end
 
-    def ensure_apply_object_class
-      current_object_class = @data['objectClass']
-      return if current_object_class.nil? or current_object_class == @last_oc
-      apply_object_class(current_object_class)
-    end
-
     # enforce_type
     #
     # enforce_type applies your changes without attempting to write to LDAP.
     # This means that if you set userCertificate to somebinary value, it will
     # wrap it up correctly.
     def enforce_type(key, value)
-      ensure_apply_object_class
       # Enforce attribute value formatting
       normalize_attribute(key, value)[1]
     end
@@ -927,60 +1055,12 @@ module ActiveLdap
       @mutex = Mutex.new
       @data = {} # where the r/w entry data is stored
       @ldap_data = {} # original ldap entry data
-      @attribute_schemata = {}
-      @attribute_names = {} # list of valid method calls for attributes used
-                            # for dereferencing
-      @normalized_attribute_names = {} # list of normalized attribute name
-      @attribute_aliases = {} # aliases of @attribute_names
-      @last_oc = false # for use in other methods for "caching"
       @dn_attribute = nil
       @base = nil
       @scope = nil
+      @dn = nil
       @connection ||= nil
-    end
-
-    # apply_object_class
-    #
-    # objectClass= special case for updating appropriately
-    # This updates the objectClass entry in @data. It also
-    # updating all required and allowed attributes while
-    # removing defined attributes that are no longer valid
-    # given the new objectclasses.
-    def apply_object_class(val)
-      new_oc = val
-      new_oc = [val] if new_oc.class != Array
-      new_oc = new_oc.uniq
-      return new_oc if @last_oc == new_oc
-
-      # Store for caching purposes
-      @last_oc = new_oc.dup
-
-      # Set the actual objectClass data
-      define_attribute_methods(schema.attribute('objectClass'))
-      replace_class(*new_oc)
-
-      # Build |data| from schema
-      # clear attribute name mapping first
-      @attribute_schemata = {}
-      @attribute_names = {}
-      @normalized_attribute_names = {}
-      @attribute_aliases = {}
-      @must = []
-      @may = []
-      @object_classes = []
-      new_oc.each do |objc|
-        # get all attributes for the class
-        object_class = schema.object_class(objc)
-        @object_classes << object_class
-        @must.concat(object_class.must)
-        @may.concat(object_class.may)
-      end
-      @must.uniq!
-      @may.uniq!
-      (@must + @may).each do |attr|
-        # Update attr_method with appropriate
-        define_attribute_methods(attr)
-      end
+      clear_connection_based_cache
     end
 
     # get_attribute
@@ -988,6 +1068,7 @@ module ActiveLdap
     # Return the value of the attribute called by method_missing?
     def get_attribute(name, force_array=false)
       name, value = get_attribute_before_type_cast(name, force_array)
+      return value if name.nil?
       attribute = schema.attribute(name)
       type_cast(attribute, value)
     end
@@ -999,7 +1080,11 @@ module ActiveLdap
         value.each do |option, val|
           result[option] = type_cast(attribute, val)
         end
-        result
+        if result.size == 1 and result.has_key?("binary")
+          result["binary"]
+        else
+          result
+        end
       when Array
         value.collect do |val|
           type_cast(attribute, val)
@@ -1013,11 +1098,7 @@ module ActiveLdap
       name = to_real_attribute_name(name)
 
       value = @data[name] || []
-      if force_array
-        [name, value.dup]
-      else
-        [name, array_of(value.dup, false)]
-      end
+      [name, array_of(value, force_array)]
     end
 
     def get_attribute_as_query(name, force_array=false)
@@ -1042,35 +1123,25 @@ module ActiveLdap
       attr, value = update_dn(attr, value) if attr == dn_attribute
       raise UnknownAttribute.new(name) if attr.nil?
 
-      case value
-      when nil, ""
-        value = []
-      when Array
-        value = value.collect {|c| c.blank? ? [] : c}.flatten
-      when String
-        value = [value]
-      when Numeric
-        value = [value.to_s]
-      end
-
-      @data[attr] = enforce_type(attr, value)
+      @data[attr] = value
     end
 
     def update_dn(attr, value)
+      @dn = nil
       @dn_is_base = false
-      return [attr, value] if value.blank?
+      return [attr, nil] if value.blank?
 
-      new_dn_attribute, new_value, base = split_dn_value(value)
+      new_dn_attribute, new_value, bases = split_dn_value(value)
       if new_dn_attribute.nil? and new_value.nil?
         @dn_is_base = true
         @base = nil
-        attr, value = DN.parse(base).rdns[0].to_a[0]
+        attr, value = bases[0].to_a[0]
         @dn_attribute = attr
       else
         new_dn_attribute = to_real_attribute_name(new_dn_attribute)
         if new_dn_attribute
           value = new_value
-          @base = base
+          @base = bases.empty? ? nil : DN.new(*bases).to_s
           if dn_attribute != new_dn_attribute
             @dn_attribute = attr = new_dn_attribute
           end
@@ -1082,13 +1153,14 @@ module ActiveLdap
     def split_dn_value(value)
       dn_value = relative_dn_value = nil
       begin
-        dn_value = DN.parse(value)
+        dn_value = value if value.is_a?(DN)
+        dn_value ||= DN.parse(value)
       rescue DistinguishedNameInvalid
         dn_value = DN.parse("#{dn_attribute}=#{value}")
       end
 
       begin
-        relative_dn_value = dn_value - DN.parse(base_of_class)
+        relative_dn_value = dn_value - self.class.parsed_base
         if relative_dn_value.rdns.empty?
           val = []
           bases = dn_value.rdns
@@ -1100,23 +1172,25 @@ module ActiveLdap
       end
 
       dn_attribute_name, dn_attribute_value = val.to_a[0]
-      [dn_attribute_name, dn_attribute_value,
-       bases.empty? ? nil : DN.new(*bases).to_s]
+      [dn_attribute_name, dn_attribute_value, bases]
     end
 
-    # define_attribute_methods
-    #
-    # Make a method entry for _every_ alias of a valid attribute and map it
-    # onto the first attribute passed in.
-    def define_attribute_methods(attribute)
-      real_name = attribute.name
-      return if @attribute_schemata.has_key?(real_name)
-      @attribute_schemata[real_name] = attribute
-      ([real_name] + attribute.aliases).each do |name|
-        @attribute_names[name] = real_name
-        @attribute_aliases[Inflector.underscore(name)] = real_name
-        @normalized_attribute_names[normalize_attribute_name(name)] = real_name
+    def compute_dn(escape_dn_value=false)
+      return base if @dn_is_base
+
+      dn_value = id
+      if dn_value.nil?
+        raise DistinguishedNameNotSetError.new,
+                _("%s's DN attribute (%s) isn't set") % [self, dn_attribute]
       end
+      dn_value = DN.escape_value(dn_value) if escape_dn_value
+      _base = base
+      _base = nil if _base.empty?
+      ["#{dn_attribute}=#{dn_value}", _base].compact.join(",")
+    end
+
+    def escaped_dn
+      compute_dn(true)
     end
 
     # array_of
@@ -1127,7 +1201,7 @@ module ActiveLdap
       case value
       when Array
         if to_a or value.size > 1
-          value.collect {|v| array_of(v, to_a)}
+          value.collect {|v| array_of(v, false)}.compact
         else
           if value.empty?
             nil
@@ -1144,7 +1218,7 @@ module ActiveLdap
           result
         end
       else
-        to_a ? [value.to_s] : value.to_s
+        to_a ? [value] : value
       end
     end
 
@@ -1158,7 +1232,7 @@ module ActiveLdap
         real_name ||= key
         next if _schema.attribute(real_name).id.nil?
         result[real_name] ||= []
-        result[real_name].concat(values)
+        result[real_name].concat(enforce_type(real_name, values))
       end
       result
     end
@@ -1201,18 +1275,18 @@ module ActiveLdap
     end
 
     def collect_all_attributes(data)
-      dn_attr = to_real_attribute_name(dn_attribute)
+      dn_attr = dn_attribute
       dn_value = data[dn_attr]
 
       attributes = []
-      attributes.push([:add, dn_attr, dn_value])
+      attributes.push([dn_attr, dn_value])
 
       oc_value = data['objectClass']
-      attributes.push([:add, 'objectClass', oc_value])
+      attributes.push(['objectClass', oc_value])
       data.each do |key, value|
         next if value.empty? or key == 'objectClass' or key == dn_attr
 
-        attributes.push([:add, key, value])
+        attributes.push([key, value])
       end
 
       attributes
@@ -1256,7 +1330,7 @@ module ActiveLdap
     def create
       prepare_data_for_saving do |data, ldap_data|
         attributes = collect_all_attributes(data)
-        add_entry(dn, attributes)
+        add_entry(escaped_dn, attributes)
         @new_entry = false
         true
       end
@@ -1265,7 +1339,7 @@ module ActiveLdap
     def update
       prepare_data_for_saving do |data, ldap_data|
         attributes = collect_modified_attributes(ldap_data, data)
-        modify_entry(dn, attributes)
+        modify_entry(escaped_dn, attributes)
         true
       end
     end
