@@ -88,7 +88,7 @@ module ActiveLdap
       end
 
       def instantiate_adapter(config)
-        adapter = (config[:adapter] || "ldap")
+        adapter = (config[:adapter] || default_adapter)
         normalized_adapter = adapter.downcase.gsub(/-/, "_")
         adapter_method = "#{normalized_adapter}_connection"
         unless Adapter::Base.respond_to?(adapter_method)
@@ -104,6 +104,10 @@ module ActiveLdap
         Adapter::Base.send(adapter_method, config)
       end
 
+      def default_adapter
+        @@default_adapter ||= guess_available_adapter
+      end
+
       def connected?
         active_connections[active_connection_name] ? true : false
       end
@@ -152,6 +156,12 @@ module ActiveLdap
         connection.schema
       end
 
+      def reset_runtime
+        active_connections.inject(0) do |result, (name, connection)|
+          result + connection.reset_runtime
+        end
+      end
+
       private
       def active_connection_key(k=self)
         k.name.empty? ? k.object_id : k.name
@@ -179,6 +189,10 @@ module ActiveLdap
         end
         @@active_connections.clear
       end
+
+      def guess_available_adapter
+        Object.respond_to?(:java) ? "jndi" : "ldap"
+      end
     end
 
     def establish_connection(config=nil)
@@ -197,8 +211,12 @@ module ActiveLdap
 
     def connection
       conn = @connection
-      if get_attribute_before_type_cast(dn_attribute)[1]
-        conn ||= self.class.active_connections[dn] || retrieve_connection
+      return conn if conn
+
+      if @dn or
+          (attribute_name_resolvable_without_connection? and
+           get_attribute_before_type_cast(dn_attribute)[1])
+        conn = self.class.active_connections[dn] || retrieve_connection
       end
       conn || self.class.connection
     end
