@@ -11,6 +11,9 @@ module ActiveLdap
       def initialize(source)
         @dn = nil
         source = source.to_s if source.is_a?(DN)
+        unless source.is_a?(String)
+          raise DistinguishedNameInputInvalid.new(source)
+        end
         @source = source
       end
 
@@ -153,13 +156,24 @@ module ActiveLdap
       def parse(source)
         Parser.new(source).parse
       end
+
+      def escape_value(value)
+        if /(\A | \z)/.match(value)
+          '"' + value.gsub(/([\\\"])/, '\\\\\1') + '"'
+        else
+          value.gsub(/([,=\+<>#;\\\"])/, '\\\\\1')
+        end
+      end
     end
 
     attr_reader :rdns
     def initialize(*rdns)
       @rdns = rdns.collect do |rdn|
-        rdn = {rdn[0] => rdn[1]} if rdn.is_a?(Array) and rdn.size == 2
-        rdn
+        if rdn.is_a?(Array) and rdn.size == 2
+          {rdn[0] => rdn[1]}
+        else
+          rdn
+        end
       end
     end
 
@@ -212,11 +226,15 @@ module ActiveLdap
         rdn.sort_by do |type, value|
           type.upcase
         end.collect do |type, value|
-          "#{type}=#{escape(value)}"
+          "#{type}=#{self.class.escape_value(value)}"
         end.join("+")
       end.join(",")
     end
 
+    def to_human_readable_format
+      to_s.inspect
+    end
+
     private
     def normalize(rdns)
       rdns.collect do |rdn|
@@ -237,14 +255,6 @@ module ActiveLdap
         [key, value]
       end
     end
-
-    def escape(value)
-      if /(\A | \z)/.match(value)
-        '"' + value.gsub(/([\\\"])/, '\\\\\1') + '"'
-      else
-        value.gsub(/([,=\+<>#;\\\"])/, '\\\\\1')
-      end
-    end
   end
 
   DN = DistinguishedName
