@@ -349,6 +349,9 @@ module ActiveLdap
     end
 
     class Attribute < Entry
+      include GetTextSupport
+      include HumanReadable
+
       attr_reader :super_attribute
       def initialize(name, schema)
         super(name, schema, "attributeTypes")
@@ -387,7 +390,7 @@ module ActiveLdap
       end
 
       def syntax
-        (@derived_syntax ||= [derived_syntax])[0]
+        @derived_syntax
       end
 
       def valid?(value)
@@ -395,7 +398,14 @@ module ActiveLdap
       end
 
       def validate(value)
-        send_to_syntax(nil, :validate, value)
+        error_info = validate_each_value(value)
+        return error_info if error_info
+        begin
+          normalize_value(value)
+          nil
+        rescue AttributeValueInvalid
+          [$!.message]
+        end
       end
 
       def type_cast(value)
@@ -403,13 +413,21 @@ module ActiveLdap
       end
 
       def normalize_value(value)
-        send_to_syntax(value, :normalize_value, value)
+        normalize_value_internal(value, false)
       end
 
       def syntax_description
         send_to_syntax(nil, :description)
       end
 
+      def human_attribute_name
+        self.class.human_attribute_name(self)
+      end
+
+      def human_attribute_description
+        self.class.human_attribute_description(self)
+      end
+
       private
       def attribute(attribute_name, name=@name)
         @schema.attribute_type(name, attribute_name)
@@ -429,18 +447,15 @@ module ActiveLdap
         if @syntax
           @binary_required = @syntax.binary_transfer_required?
           @binary = (@binary_required or !@syntax.human_readable?)
+          @derived_syntax = @syntax
         else
           @binary_required = false
           @binary = false
+          @derived_syntax = nil
+          @derived_syntax = @super_attribute.syntax if @super_attribute
         end
       end
 
-      def derived_syntax
-        return @syntax if @syntax
-        return @super_attribute.syntax if @super_attribute
-        nil
-      end
-
       def send_to_syntax(default_value, method_name, *args)
         _syntax = syntax
         if _syntax
@@ -449,6 +464,100 @@ module ActiveLdap
           default_value
         end
       end
+
+      def validate_each_value(value, option=nil)
+        failed_reason = nil
+        case value
+        when Hash
+          original_option = option
+          value.each do |sub_option, val|
+            opt = [original_option, sub_option].compact.join(";")
+            failed_reason, option = validate_each_value(val, opt)
+            break if failed_reason
+          end
+        when Array
+          original_option = option
+          value.each do |val|
+            failed_reason, option = validate_each_value(val, original_option)
+            break if failed_reason
+          end
+        else
+          failed_reason = send_to_syntax(nil, :validate, value)
+        end
+        return nil if failed_reason.nil?
+        [failed_reason, option]
+      end
+
+      def normalize_value_internal(value, have_binary_mark)
+        case value
+        when Array
+          normalize_array_value(value, have_binary_mark)
+        when Hash
+          normalize_hash_value(value, have_binary_mark)
+        else
+          if value.blank?
+            value = []
+          else
+            value = send_to_syntax(value, :normalize_value, value)
+          end
+          if !have_binary_mark and binary_required?
+            [{'binary' => value}]
+          else
+            value.is_a?(Array) ? value : [value]
+          end
+        end
+      end
+
+      def normalize_array_value(value, have_binary_mark)
+        if single_value? and value.reject {|v| v.is_a?(Hash)}.size > 1
+          format = _("Attribute %s can only have a single value: %s")
+          message = format % [human_attribute_name, value.inspect]
+          raise AttributeValueInvalid.new(self, value, message)
+        end
+        if value.empty?
+          if !have_binary_mark and binary_required?
+            [{'binary' => value}]
+          else
+            value
+          end
+        else
+          value.collect do |entry|
+            normalize_value_internal(entry, have_binary_mark)[0]
+          end
+        end
+      end
+
+      def normalize_hash_value(value, have_binary_mark)
+        if value.size > 1
+          format = _("Attribute %s: Hash must have one key-value pair only: %s")
+          message = format % [human_attribute_name, value.inspect]
+          raise AttributeValueInvalid.new(self, value, message)
+        end
+
+        if !have_binary_mark and binary_required? and !have_binary_key?(value)
+          [append_binary_key(value)]
+        else
+          key = value.keys[0]
+          have_binary_mark ||= key == "binary"
+          [{key => normalize_value_internal(value.values[0], have_binary_mark)}]
+        end
+      end
+
+      def have_binary_key?(hash)
+        key, value = hash.to_a[0]
+        return true if key == "binary"
+        return have_binary_key?(value) if value.is_a?(Hash)
+        false
+      end
+
+      def append_binary_key(hash)
+        key, value = hash.to_a[0]
+        if value.is_a?(Hash)
+          append_binary_key(value)
+        else
+          hash.merge(key => {"binary" => value})
+        end
+      end
     end
 
     class ObjectClass < Entry
