@@ -1,7 +1,7 @@
 require_library_or_gem 'active_ldap'
 ActiveLdap::Base.logger ||= RAILS_DEFAULT_LOGGER
 
-required_version = ["0", "8", "4"]
+required_version = ["1", "0", "0"]
 if (ActiveLdap::VERSION.split(".") <=> required_version) < 0
   ActiveLdap::Base.class_eval do
     format = _("You need ActiveLdap %s or later")
@@ -21,6 +21,13 @@ else
   end
 end
 
-class ActionView::Base
+class ::ActionView::Base
   include ActiveLdap::Helper
 end
+
+require 'active_ldap/action_controller/ldap_benchmarking'
+module ::ActionController
+  class Base
+    include ActiveLdap::ActionController::LdapBenchmarking
+  end
+end
