@@ -1,16 +1,19 @@
 #!/usr/bin/env ruby
 
+$VERBOSE = true
+
 $KCODE = 'u'
 
 require 'yaml'
-require "test/unit"
 
 base_dir = File.expand_path(File.dirname(__FILE__))
 top_dir = File.expand_path(File.join(base_dir, ".."))
 $LOAD_PATH.unshift(File.join(top_dir, "lib"))
 $LOAD_PATH.unshift(File.join(top_dir, "test"))
 
-require 'test-unit-ext'
+$LOAD_PATH.unshift(File.join(top_dir, "test-unit", "lib"))
+require "test/unit"
+ARGV.unshift("--priority-mode")
 
 test_file = "test/test_*.rb"
 Dir.glob(test_file) do |file|
