@@ -10,6 +10,19 @@ class TestAdapter < Test::Unit::TestCase
   end
 
   priority :must
+  def test_operator
+    assert_parse_filter("(uid~=Alice)", ["uid", "~=", "Alice"])
+    assert_parse_filter("(&(uid~=Alice)(uid~=Bob))",
+                        ["uid", "~=", "Alice", "Bob"])
+    assert_parse_filter("(uid~=Alice)", [["uid", "~=", "Alice"]])
+    assert_parse_filter("(|(uid~=Alice)(uid~=Bob))",
+                        [:or,
+                         ["uid", "~=", "Alice"],
+                         ["uid", "~=", "Bob"]])
+    assert_parse_filter("(|(uid~=Alice)(uid~=Bob))",
+                        [:or,
+                         ["uid", "~=", "Alice", "Bob"]])
+  end
 
   priority :normal
   def test_filter_with_escaped_character
@@ -73,14 +86,20 @@ class TestAdapter < Test::Unit::TestCase
     assert_parse_filter("(&(gidNumber=100001)" +
                         "(|(uid=temp-user1)(uid=temp-user2)))",
                         [:and,
-                         [:and, {"gidNumber"=>["100001"]}],
-                         [:or, {"uid"=>["temp-user1", "temp-user2"]}]])
+                         [:and, {"gidNumber" => ["100001"]}],
+                         [:or, {"uid" => ["temp-user1", "temp-user2"]}]])
     assert_parse_filter("(&(gidNumber=100001)" +
                         "(objectClass=person)(objectClass=posixAccount))",
                         [:and,
                          [:or, ["gidNumber", "100001"]],
                          ["objectClass", "person"],
                          ["objectClass", "posixAccount"]])
+    assert_parse_filter("(&(!(|(gidNumber=100001)(gidNumber=100002)))" +
+                        "(objectClass=person)(!(objectClass=posixAccount)))",
+                        [:and,
+                         [:not, [:or, ["gidNumber", "100001", "100002"]]],
+                         ["objectClass", "person"],
+                         [:not, ["objectClass", "posixAccount"]]])
   end
 
   def test_invalid_operator
