@@ -4,20 +4,117 @@ class TestAssociations < Test::Unit::TestCase
   include AlTestUtils
 
   priority :must
+  def test_has_many_wrap_with_dn_value
+    @user_class.has_many :references, :wrap => "seeAlso", :primary_key => "dn"
+    @user_class.set_associated_class(:references, @group_class)
+    @group_class.belongs_to :related_users, :many => "seeAlso",
+                            :foreign_key => "dn"
+    @group_class.set_associated_class(:related_users, @user_class)
+    make_temporary_user do |user,|
+      make_temporary_group do |group1|
+        make_temporary_group do |group2|
+          make_temporary_group do |group3|
+            entries = [user, group1, group2, group3]
+
+            user.references << group1
+            user, group1, group2, group3 = reload_entries(*entries)
+            assert_references([[group1]], [user])
+            assert_related_users([user], group1)
+            assert_related_users([], group2)
+            assert_related_users([], group3)
+
+            user.references = [group2, group3]
+            user, group1, group2, group3 = reload_entries(*entries)
+            assert_references([[group2, group3]], [user])
+            assert_related_users([], group1)
+            assert_related_users([user], group2)
+            assert_related_users([user], group3)
+
+            user.references.delete(group2)
+            user, group1, group2, group3 = reload_entries(*entries)
+            assert_references([[group3]], [user])
+            assert_related_users([], group1)
+            assert_related_users([], group2)
+            assert_related_users([user], group3)
+          end
+        end
+      end
+    end
+  end
 
   priority :normal
-  def test_belongs_to_many_with_dn_key
-    @user_class.belongs_to :groups, :many => "memberUid", :foreign_key => "dn"
-    @user_class.set_associated_class(:groups, @group_class)
-    @group_class.has_many :members, :wrap => "memberUid", :primary_key => "dn"
-    @group_class.set_associated_class(:members, @user_class)
+  def test_belongs_to_many_with_dn_value
+    @user_class.has_many :references, :wrap => "seeAlso", :primary_key => "dn"
+    @user_class.set_associated_class(:references, @group_class)
+    @group_class.belongs_to :related_users, :many => "seeAlso",
+                            :foreign_key => "dn"
+    @group_class.set_associated_class(:related_users, @user_class)
     make_temporary_group do |group|
       make_temporary_user do |user1,|
         make_temporary_user do |user2,|
-          group.members = [user1, user2]
-          assert(group.save)
-          assert_equal([group.cn], user1.groups.collect(&:cn))
-          assert_equal([group.cn], user2.groups.collect(&:cn))
+          make_temporary_user do |user3,|
+            entries = [group, user1, user2, user3]
+
+            group.related_users = [user1, user2]
+            group, user1, user2, user3 = reload_entries(*entries)
+            assert_references([[group], [group], []],
+                              [user1, user2, user3])
+            assert_related_users([user1, user2], group)
+
+            group.related_users << user3
+            group, user1, user2, user3 = reload_entries(*entries)
+            assert_references([[group], [group], [group]],
+                              [user1, user2, user3])
+            assert_related_users([user1, user2, user3], group)
+
+            group.related_users.delete(user1)
+            group, user1, user2, user3 = reload_entries(*entries)
+            assert_references([[], [group], [group]],
+                              [user1, user2, user3])
+            assert_related_users([user2, user3], group)
+
+            group.related_users = []
+            group, user1, user2, user3 = reload_entries(*entries)
+            assert_references([[], [], []],
+                              [user1, user2, user3])
+            assert_related_users([], group)
+          end
+        end
+      end
+    end
+  end
+
+  def test_belongs_to_many_with_dn_key
+    @user_class.belongs_to :dn_groups, :many => "memberUid", :foreign_key => "dn"
+    @user_class.set_associated_class(:dn_groups, @group_class)
+    @group_class.has_many :dn_members, :wrap => "memberUid", :primary_key => "dn"
+    @group_class.set_associated_class(:dn_members, @user_class)
+    make_temporary_group do |group|
+      make_temporary_user do |user1,|
+        make_temporary_user do |user2,|
+          make_temporary_user do |user3,|
+            entries = [group, user1, user2, user3]
+
+            user1.dn_groups << group
+            group, user1, user2, user3 = reload_entries(*entries)
+            assert_dn_groups([[group], [], []], [user1, user2, user3])
+            assert_dn_members([user1], group)
+
+            user2.dn_groups = [group]
+            group, user1, user2, user3 = reload_entries(*entries)
+            assert_dn_groups([[group], [group], []], [user1, user2, user3])
+            assert_dn_members([user1, user2], group)
+
+            user1.dn_groups = []
+            group, user1, user2, user3 = reload_entries(*entries)
+            assert_dn_groups([[], [group], []], [user1, user2, user3])
+            assert_dn_members([user2], group)
+
+            user2.dn_groups.delete(group)
+            group, user1, user2, user3 = reload_entries(*entries)
+            assert_dn_groups([[], [], []], [user1, user2, user3])
+            assert_dn_members([], group)
+          end
         end
       end
     end
@@ -44,7 +141,6 @@ class TestAssociations < Test::Unit::TestCase
     end
   end
 
-  priority :normal
   def test_belongs_to_before_save
     make_temporary_group do |group1|
       make_temporary_group do |group2|
@@ -350,4 +446,42 @@ class TestAssociations < Test::Unit::TestCase
       end
     end
   end
+
+  private
+  def reload_entries(*entries)
+    entries.collect do |entry|
+      entry.class.find(entry[entry.dn_attribute])
+    end
+  end
+
+  def assert_groups_relation(expected_groups_values, entries, relation_name)
+    expected_groups_values = expected_groups_values.collect do |groups|
+      groups.collect(&:cn).sort
+    end
+    actual_groups_values = entries.collect do |entry|
+      entry.send(relation_name).collect(&:cn).sort
+    end
+    assert_equal(expected_groups_values, actual_groups_values)
+  end
+
+  def assert_users_relation(expected_users, group, relation_name)
+    assert_equal(expected_users.collect(&:cn).sort,
+                 group.send(relation_name).collect(&:cn).sort)
+  end
+
+  def assert_references(expected_groups_values, users)
+    assert_groups_relation(expected_groups_values, users, :references)
+  end
+
+  def assert_related_users(expected_users, group)
+    assert_users_relation(expected_users, group, :related_users)
+  end
+
+  def assert_dn_groups(expected_groups_values, users)
+    assert_groups_relation(expected_groups_values, users, :dn_groups)
+  end
+
+  def assert_dn_members(expected_users, group)
+    assert_users_relation(expected_users, group, :dn_members)
+  end
 end
