@@ -1,11 +1,91 @@
+# -*- coding: utf-8 -*-
+
 require 'al-test-utils'
 
 class TestBase < Test::Unit::TestCase
   include AlTestUtils
 
   priority :must
+  def test_to_ldif
+    make_temporary_group do |group,|
+      assert_to_ldif(group)
+
+      group.gidNumber += 1
+      group.description = ["Description", {"en" => "Description(en)"}]
+      assert_to_ldif(group)
+    end
+  end
 
   priority :normal
+  def test_save_with_changes
+    make_temporary_user do |user, password|
+      user.cn += "!!!"
+      assert_true(detect_modify(user) {user.save})
+    end
+  end
+
+  def test_save_without_changes
+    make_temporary_user do |user, password|
+      assert_false(detect_modify(user) {user.save})
+    end
+  end
+
+  def test_normalize_dn_attribute
+    make_ou("Ous")
+    ou_class = Class.new(ActiveLdap::Base)
+    ou_class.ldap_mapping(:dn_attribute => "OU",
+                          :prefix => "ou=OUS",
+                          :classes => ["top", "organizationalUnit"])
+    ou_class.new("ou1").save!
+    ou_class.new("ou2").save!
+
+    ou1 = ou_class.find("ou1")
+    assert_equal("ou1", ou1.ou)
+    assert_equal("ou=ou1,#{ou_class.base}", ou1.dn)
+    ou2 = ou_class.find("ou2")
+    assert_equal("ou2", ou2.ou)
+    assert_equal("ou=ou2,#{ou_class.base}", ou2.dn)
+  end
+
+  def test_excluded_classes
+    mapping = {:classes => ["person"]}
+    person_class = Class.new(@user_class)
+    person_class.ldap_mapping(mapping)
+
+    no_organizational_person_class = Class.new(@user_class)
+    no_organizational_person_mapping =
+      mapping.merge(:excluded_classes => ["organizationalPerson"])
+    no_organizational_person_class.ldap_mapping(no_organizational_person_mapping)
+
+    no_simple_person_class = Class.new(@user_class)
+    no_simple_person_mapping =
+      mapping.merge(:excluded_classes => ['shadowAccount', 'inetOrgPerson',
+                                          "organizationalPerson"])
+    no_simple_person_class.ldap_mapping(no_simple_person_mapping)
+
+    make_temporary_user do |user1,|
+      make_temporary_user(:simple => true) do |user2,|
+        assert_equal([user1.dn, user2.dn].sort,
+                     person_class.find(:all).collect(&:dn).sort)
+
+        no_organizational_people = no_organizational_person_class.find(:all)
+        assert_equal([user2.dn].sort,
+                     no_organizational_people.collect(&:dn).sort)
+
+        assert_equal([user2.dn].sort,
+                     no_simple_person_class.find(:all).collect(&:dn).sort)
+      end
+    end
+  end
+
+  def test_new_with_dn
+    cn = "XXX"
+    dn = "cn=#{cn},#{@user_class.base}"
+    user = @user_class.new(ActiveLdap::DN.parse(dn))
+    assert_equal(cn, user.cn)
+    assert_equal(dn, user.dn)
+  end
+
   def test_dn_attribute_per_instance_with_invalid_value
     user = @user_class.new
     assert_equal("uid", user.dn_attribute)
@@ -134,6 +214,15 @@ class TestBase < Test::Unit::TestCase
                  ou_class.search(:value => name).collect {|dn, attrs| dn})
   end
 
+  def test_search_with_attributes_without_object_class
+    make_temporary_user do |user, password|
+      entries = @user_class.search(:filter => "#{user.dn_attribute}=#{user.id}",
+                                   :attributes => ["uidNumber"])
+      assert_equal([[user.dn, {"uidNumber" => [user.uid_number.to_s]}]],
+                    entries)
+    end
+  end
+
   def test_new_without_argument
     user = @user_class.new
     assert_equal(@user_class_classes, user.classes)
@@ -343,7 +432,9 @@ class TestBase < Test::Unit::TestCase
                    user2.attributes.reject {|k, v| k == "cn"})
 
       user2.reload
-      assert_equal(user1.attributes, user2.attributes)
+      assert_equal(user1.cn, user2.cn)
+      assert_equal(user1.attributes.reject {|k, v| k == "cn"},
+                   user2.attributes.reject {|k, v| k == "cn"})
     end
   end
 
@@ -409,6 +500,16 @@ class TestBase < Test::Unit::TestCase
     end
   end
 
+  def test_ldap_mapping_symbol_dn_attribute
+    ou_class = Class.new(ActiveLdap::Base)
+    ou_class.ldap_mapping(:dn_attribute => :ou,
+                          :prefix => "",
+                          :classes => ["top", "organizationalUnit"])
+    assert_equal(["ou=Groups,#{current_configuration['base']}",
+                  "ou=Users,#{current_configuration['base']}"],
+                 ou_class.find(:all).collect(&:dn).sort)
+  end
+
   def test_ldap_mapping_validation
     ou_class = Class.new(ActiveLdap::Base)
     assert_raises(ArgumentError) do
@@ -504,6 +605,12 @@ EOX
     end
   end
 
+  def test_attribute_present_with_unknown_attribute
+    make_temporary_user do |user, password|
+      assert(!user.attribute_present?(:unknown_attribute))
+    end
+  end
+
   def test_update_all
     make_temporary_user do |user, password|
       make_temporary_user do |user2, password|
@@ -544,4 +651,33 @@ EOX
       end
     end
   end
+
+  private
+  def detect_modify(object)
+    modify_called = false
+    singleton_class = class << object; self; end
+    singleton_class.send(:define_method, :modify_entry) do |*args|
+      dn, attributes, options = args
+      options ||= {}
+      modify_detector = Object.new
+      modify_detector.instance_variable_set("@called", false)
+      def modify_detector.modify(dn, entries, options)
+        @called = true
+      end
+      options[:connection] = modify_detector
+      result = super(dn, attributes, options)
+      modify_called = modify_detector.instance_variable_get("@called")
+      result
+    end
+    yield
+    modify_called
+  end
+
+  def assert_to_ldif(entry)
+    records = ActiveLdap::LDIF.parse(entry.to_ldif).records
+    parsed_entries = records.collect do |record|
+      entry.class.send(:instantiate, [record.dn, record.attributes])
+    end
+    assert_equal([entry], parsed_entries)
+  end
 end
