@@ -9,6 +9,19 @@ class TestBasePerInstance < Test::Unit::TestCase
   end
 
   priority :must
+  def test_dn_is_base
+    entry_class = Class.new(ActiveLdap::Base)
+    entry_class.ldap_mapping :prefix => "",
+                             :classes => ["top"],
+                             :scope => :sub
+    entry_class.dn_attribute = nil
+
+    entry = entry_class.root
+    assert_equal(entry_class.base, entry.dn)
+    assert_equal(entry_class.base, entry.base)
+  end
+
+  priority :normal
   def test_loose_dn
     user = @user_class.new("test-user , ou = Sub")
     assert_equal("uid=test-user,ou=Sub,#{@user_class.base}", user.dn)
@@ -17,7 +30,6 @@ class TestBasePerInstance < Test::Unit::TestCase
     assert_equal("uid=test-user,ou=Sub,#{@user_class.base}", user.dn)
   end
 
-  priority :normal
   def test_exists?
     make_temporary_user(:uid => "test-user,ou=Sub") do |user, password|
       assert(@user_class.exists?(user.uid))
